import * as utils from './index';

var MONTH_REGEX = /(0[1-9]|1[0-2])/;

export var EMPTY_CARD_NUMBER = 'Enter a card number';
export var EMPTY_EXPIRY_DATE = 'Enter an expiry date';
export var EMPTY_CVC = 'Enter a CVC';
export var EMPTY_ZIP = 'Enter a ZIP code';

export var INVALID_CARD_NUMBER = 'Card number is invalid';
export var INVALID_EXPIRY_DATE = 'Expiry date is invalid';
export var INVALID_CVC = 'CVC is invalid';

export var MONTH_OUT_OF_RANGE = 'Expiry month must be between 01 and 12';
export var YEAR_OUT_OF_RANGE = 'Expiry year cannot be in the past';
export var DATE_OUT_OF_RANGE = 'Expiry date cannot be in the past';

export var hasCardNumberReachedMaxLength = function hasCardNumberReachedMaxLength(currentValue) {
  var cardType = utils.cardTypes.getCardTypeByValue(currentValue);
  return cardType && currentValue.length >= cardType.lengths[cardType.lengths.length - 1];
};

export var isNumeric = function isNumeric(e) {
  return (/^\d*$/.test(e.key)
  );
};

export var validateLuhn = function validateLuhn(cardNumber) {
  return cardNumber.split('').reverse().map(function (digit) {
    return parseInt(digit, 10);
  }).map(function (digit, idx) {
    return idx % 2 ? digit * 2 : digit;
  }).map(function (digit) {
    return digit > 9 ? digit % 10 + 1 : digit;
  }).reduce(function (accum, digit) {
    return accum += digit;
  }) % 10 === 0;
};
export var getCardNumberError = function getCardNumberError(cardNumber) {
  if (!cardNumber) {
    return EMPTY_CARD_NUMBER;
  }

  var cardType = utils.cardTypes.getCardTypeByValue(cardNumber);

  if (cardType && cardType.lengths) {
    var doesCardNumberMatchLength = cardType.lengths.includes(cardNumber.length);
    if (doesCardNumberMatchLength) {
      var isLuhnValid = validateLuhn(cardNumber);
      if (isLuhnValid) {
        return;
      }
    }
  }

  return INVALID_CARD_NUMBER;
};
export var getExpiryDateError = function getExpiryDateError(expiryDate) {
  if (!expiryDate) {
    return EMPTY_EXPIRY_DATE;
  }
  var rawExpiryDate = expiryDate.replace(' / ', '');
  if (rawExpiryDate.length === 4) {
    var month = rawExpiryDate.slice(0, 2);
    var year = '20' + rawExpiryDate.slice(2, 4);
    if (!MONTH_REGEX.test(month)) {
      return MONTH_OUT_OF_RANGE;
    }
    if (parseInt(year) < new Date().getFullYear()) {
      return YEAR_OUT_OF_RANGE;
    }
    if (parseInt(year) === new Date().getFullYear() && parseInt(month) < new Date().getMonth() + 1) {
      return DATE_OUT_OF_RANGE;
    }
    return;
  }
  return INVALID_EXPIRY_DATE;
};
export var getCVCError = function getCVCError(expiryDate, _ref) {
  var cardType = _ref.cardType;

  if (!expiryDate) {
    return EMPTY_EXPIRY_DATE;
  }
  if (cardType && expiryDate.length !== cardType.code.length) {
    return INVALID_CVC;
  }
  return;
};
export var getZIPError = function getZIPError(zip) {
  if (!zip) {
    return EMPTY_ZIP;
  }
  return;
};