'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = usePaymentCard;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utils = require('./utils');

var utils = _interopRequireWildcard(_utils);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * TODO:
 *  - test default value errors
 *  - unified error message
 *  - unified touch state
 *  - handle images
 *  - wrapper with default styling
 */

function usePaymentCard() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      onError = _ref.onError;

  var cardNumberField = _react2.default.useRef();
  var expiryDateField = _react2.default.useRef();
  var cvcField = _react2.default.useRef();
  var zipField = _react2.default.useRef();

  /** ====== START: META STUFF ====== */

  var _React$useState = _react2.default.useState({
    cardNumber: false,
    expiryDate: false,
    cvc: false,
    zip: false
  }),
      touchedInputs = _React$useState[0],
      setTouchedInputs = _React$useState[1];

  var _React$useState2 = _react2.default.useState(false),
      isTouched = _React$useState2[0],
      setIsTouched = _React$useState2[1];

  var _React$useState3 = _react2.default.useState({
    cardNumber: utils.validator.EMPTY_CARD_NUMBER,
    expiryDate: utils.validator.EMPTY_EXPIRY_DATE,
    cvc: utils.validator.EMPTY_CVC,
    zip: utils.validator.EMPTY_ZIP
  }),
      errors = _React$useState3[0],
      setErrors = _React$useState3[1];

  var _React$useState4 = _react2.default.useState(),
      error = _React$useState4[0],
      setError = _React$useState4[1];

  var _React$useState5 = _react2.default.useState(),
      cardType = _React$useState5[0],
      setCardType = _React$useState5[1];
  /** ====== END: META STUFF ====== */

  /** ====== START: CARD NUMBER STUFF ====== */


  var handleBlurCardNumber = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      props.onBlur && props.onBlur(e);

      setTouchedInputs(function (touchedInputs) {
        return _extends({}, touchedInputs, { cardNumber: true });
      });
    };
  }, []);

  var handleChangeCardNumber = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      var formattedCardNumber = e.target.value || '';
      var cardNumber = formattedCardNumber.replace(/\s/g, '');

      var cardType = utils.cardTypes.getCardTypeByValue(cardNumber);
      setCardType(cardType);

      props.onChange && props.onChange(e);

      // @ts-ignore
      cardNumberField.current.value = utils.formatter.formatCardNumber(cardNumber);

      var cardNumberError = utils.validator.getCardNumberError(cardNumber);

      if (!cardNumberError) {
        expiryDateField.current && expiryDateField.current.focus();
      }

      setErrors(function (errors) {
        var newErrors = _extends({}, errors, { cardNumber: cardNumberError });
        onError && onError({ cardNumber: cardNumberError }, newErrors);
        return newErrors;
      });
    };
  }, [cardNumberField]);

  var handleKeyPressCardNumber = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      var formattedCardNumber = e.target.value || '';
      var cardNumber = formattedCardNumber.replace(/\s/g, '');

      props.onKeyPress && props.onKeyPress(e);

      if (!utils.validator.isNumeric(e)) {
        e.preventDefault();
      }
      if (utils.validator.hasCardNumberReachedMaxLength(cardNumber)) {
        e.preventDefault();
      }
    };
  }, []);

  var cardNumberProps = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    return {
      autoComplete: 'cc-number',
      name: 'cardnumber',
      placeholder: 'Card number',
      type: 'tel',
      ref: cardNumberField,
      onBlur: handleBlurCardNumber(props),
      onChange: handleChangeCardNumber(props),
      onKeyPress: handleKeyPressCardNumber(props)
    };
  }, [cardNumberField]);
  /** ====== END: CARD NUMBER STUFF ====== */

  /** ====== START: EXPIRY DATE STUFF ====== */
  var handleBlurExpiryDate = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      props.onBlur && props.onBlur(e);

      setTouchedInputs(function (touchedInputs) {
        return _extends({}, touchedInputs, { expiryDate: true });
      });
    };
  }, []);

  var handleChangeExpiryDate = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      var expiryDate = e.target.value;

      props.onChange && props.onChange(e);

      expiryDateField.current.value = utils.formatter.formatExpiry(e);

      var expiryDateError = utils.validator.getExpiryDateError(expiryDate);

      if (!expiryDateError) {
        cvcField.current && cvcField.current.focus();
      }

      setErrors(function (errors) {
        var newErrors = _extends({}, errors, { expiryDate: expiryDateError });
        onError && onError({ expiryDate: expiryDateError }, newErrors);
        return newErrors;
      });
    };
  }, [expiryDateField]);

  var handleKeyDownExpiryDate = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      props.onKeyDown && props.onKeyDown(e);

      if (e.keyCode === utils.BACKSPACE_KEY_CODE && !e.target.value) {
        cardNumberField.current && cardNumberField.current.focus();
      }
    };
  }, [cardNumberField]);

  var handleKeyPressExpiryDate = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      var formattedExpiryDate = e.target.value || '';
      var expiryDate = formattedExpiryDate.replace(' / ', '');

      props.onKeyPress && props.onKeyPress(e);

      if (!utils.validator.isNumeric(e)) {
        e.preventDefault();
      }
      if (expiryDate.length >= 4) {
        e.preventDefault();
      }
    };
  }, []);

  var expiryDateProps = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    return {
      autoComplete: 'cc-exp',
      name: 'cc-exp',
      placeholder: 'MM/YY',
      type: 'tel',
      ref: expiryDateField,
      onBlur: handleBlurExpiryDate(props),
      onChange: handleChangeExpiryDate(props),
      onKeyDown: handleKeyDownExpiryDate(props),
      onKeyPress: handleKeyPressExpiryDate(props)
    };
  }, [expiryDateField]);
  /** ====== END: EXPIRY DATE STUFF ====== */

  /** ====== START: CVC STUFF ====== */
  var handleBlurCVC = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      props.onBlur && props.onBlur(e);

      setTouchedInputs(function (touchedInputs) {
        return _extends({}, touchedInputs, { cvc: true });
      });
    };
  }, []);

  var handleChangeCVC = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var _ref2 = arguments[1];
    var cardType = _ref2.cardType;

    return function (e) {
      var cvc = e.target.value;

      props.onChange && props.onChange(e);

      var cvcError = utils.validator.getCVCError(cvc, { cardType: cardType });

      if (!cvcError) {
        zipField.current && zipField.current.focus();
      }

      setErrors(function (errors) {
        var newErrors = _extends({}, errors, { cvc: cvcError });
        onError && onError({ cvc: cvcError }, newErrors);
        return newErrors;
      });
    };
  }, [cvcField]);

  var handleKeyDownCVC = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      props.onKeyDown && props.onKeyDown(e);

      if (e.keyCode === utils.BACKSPACE_KEY_CODE && !e.target.value) {
        expiryDateField.current && expiryDateField.current.focus();
      }
    };
  }, [expiryDateField]);

  var handleKeyPressCVC = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var _ref3 = arguments[1];
    var cardType = _ref3.cardType;

    return function (e) {
      var formattedCVC = e.target.value || '';
      var cvc = formattedCVC.replace(' / ', '');

      props.onKeyPress && props.onKeyPress(e);

      if (!utils.validator.isNumeric(e)) {
        e.preventDefault();
      }
      if (cardType && cvc.length >= cardType.code.length) {
        e.preventDefault();
      }
      if (cvc.length >= 4) {
        e.preventDefault();
      }
    };
  }, []);

  var cvcProps = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    return {
      autoComplete: 'cc-csc',
      name: 'cvc',
      placeholder: cardType ? cardType.code.name : 'CVC',
      type: 'tel',
      ref: cvcField,
      onBlur: handleBlurCVC(props),
      onChange: handleChangeCVC(props, { cardType: cardType }),
      onKeyDown: handleKeyDownCVC(props),
      onKeyPress: handleKeyPressCVC(props, { cardType: cardType })
    };
  }, [cvcField, cardType]);
  /** ====== END: CVC STUFF ====== */

  /** ====== START: ZIP STUFF ====== */
  var handleBlurZIP = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      props.onBlur && props.onBlur(e);

      setTouchedInputs(function (touchedInputs) {
        return _extends({}, touchedInputs, { zip: true });
      });
    };
  }, []);

  var handleChangeZIP = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      var zip = e.target.value;

      props.onChange && props.onChange(e);

      var zipError = utils.validator.getZIPError(zip);
      setErrors(function (errors) {
        var newErrors = _extends({}, errors, { zip: zipError });
        onError && onError({ zip: zipError }, newErrors);
        return newErrors;
      });
    };
  }, [zipField]);

  var handleKeyDownZIP = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      props.onKeyDown && props.onKeyDown(e);

      if (e.keyCode === utils.BACKSPACE_KEY_CODE && !e.target.value) {
        cvcField.current && cvcField.current.focus();
      }
    };
  }, [cvcField]);

  var handleKeyPressZIP = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return function (e) {
      props.onKeyPress && props.onKeyPress(e);

      if (!utils.validator.isNumeric(e)) {
        e.preventDefault();
      }
    };
  }, []);

  var zipProps = _react2.default.useCallback(function () {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    return {
      autoComplete: 'off',
      maxLength: '6',
      name: 'zip',
      placeholder: 'ZIP',
      type: 'tel',
      ref: zipField,
      onBlur: handleBlurZIP(props),
      onChange: handleChangeZIP(props),
      onKeyDown: handleKeyDownZIP(props),
      onKeyPress: handleKeyPressZIP(props)
    };
  }, [zipField]);
  /** ====== END: ZIP STUFF ====== */

  return {
    cardType: cardType,
    errors: errors,
    error: error,
    isTouched: isTouched,
    touchedInputs: touchedInputs,

    cardNumberProps: cardNumberProps,
    expiryDateProps: expiryDateProps,
    cvcProps: cvcProps,
    zipProps: zipProps
  };
}
module.exports = exports['default'];