'use strict';

exports.__esModule = true;
exports.getZIPError = exports.getCVCError = exports.getExpiryDateError = exports.getCardNumberError = exports.validateLuhn = exports.isNumeric = exports.hasCardNumberReachedMaxLength = exports.DATE_OUT_OF_RANGE = exports.YEAR_OUT_OF_RANGE = exports.MONTH_OUT_OF_RANGE = exports.INVALID_CVC = exports.INVALID_EXPIRY_DATE = exports.INVALID_CARD_NUMBER = exports.EMPTY_ZIP = exports.EMPTY_CVC = exports.EMPTY_EXPIRY_DATE = exports.EMPTY_CARD_NUMBER = undefined;

var _index = require('./index');

var utils = _interopRequireWildcard(_index);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var MONTH_REGEX = /(0[1-9]|1[0-2])/;

var EMPTY_CARD_NUMBER = exports.EMPTY_CARD_NUMBER = 'Enter a card number';
var EMPTY_EXPIRY_DATE = exports.EMPTY_EXPIRY_DATE = 'Enter an expiry date';
var EMPTY_CVC = exports.EMPTY_CVC = 'Enter a CVC';
var EMPTY_ZIP = exports.EMPTY_ZIP = 'Enter a ZIP code';

var INVALID_CARD_NUMBER = exports.INVALID_CARD_NUMBER = 'Card number is invalid';
var INVALID_EXPIRY_DATE = exports.INVALID_EXPIRY_DATE = 'Expiry date is invalid';
var INVALID_CVC = exports.INVALID_CVC = 'CVC is invalid';

var MONTH_OUT_OF_RANGE = exports.MONTH_OUT_OF_RANGE = 'Expiry month must be between 01 and 12';
var YEAR_OUT_OF_RANGE = exports.YEAR_OUT_OF_RANGE = 'Expiry year cannot be in the past';
var DATE_OUT_OF_RANGE = exports.DATE_OUT_OF_RANGE = 'Expiry date cannot be in the past';

var hasCardNumberReachedMaxLength = exports.hasCardNumberReachedMaxLength = function hasCardNumberReachedMaxLength(currentValue) {
  var cardType = utils.cardTypes.getCardTypeByValue(currentValue);
  return cardType && currentValue.length >= cardType.lengths[cardType.lengths.length - 1];
};

var isNumeric = exports.isNumeric = function isNumeric(e) {
  return (/^\d*$/.test(e.key)
  );
};

var validateLuhn = exports.validateLuhn = function validateLuhn(cardNumber) {
  return cardNumber.split('').reverse().map(function (digit) {
    return parseInt(digit, 10);
  }).map(function (digit, idx) {
    return idx % 2 ? digit * 2 : digit;
  }).map(function (digit) {
    return digit > 9 ? digit % 10 + 1 : digit;
  }).reduce(function (accum, digit) {
    return accum += digit;
  }) % 10 === 0;
};
var getCardNumberError = exports.getCardNumberError = function getCardNumberError(cardNumber) {
  if (!cardNumber) {
    return EMPTY_CARD_NUMBER;
  }

  var cardType = utils.cardTypes.getCardTypeByValue(cardNumber);

  if (cardType && cardType.lengths) {
    var doesCardNumberMatchLength = cardType.lengths.includes(cardNumber.length);
    if (doesCardNumberMatchLength) {
      var isLuhnValid = validateLuhn(cardNumber);
      if (isLuhnValid) {
        return;
      }
    }
  }

  return INVALID_CARD_NUMBER;
};
var getExpiryDateError = exports.getExpiryDateError = function getExpiryDateError(expiryDate) {
  if (!expiryDate) {
    return EMPTY_EXPIRY_DATE;
  }
  var rawExpiryDate = expiryDate.replace(' / ', '');
  if (rawExpiryDate.length === 4) {
    var month = rawExpiryDate.slice(0, 2);
    var year = '20' + rawExpiryDate.slice(2, 4);
    if (!MONTH_REGEX.test(month)) {
      return MONTH_OUT_OF_RANGE;
    }
    if (parseInt(year) < new Date().getFullYear()) {
      return YEAR_OUT_OF_RANGE;
    }
    if (parseInt(year) === new Date().getFullYear() && parseInt(month) < new Date().getMonth() + 1) {
      return DATE_OUT_OF_RANGE;
    }
    return;
  }
  return INVALID_EXPIRY_DATE;
};
var getCVCError = exports.getCVCError = function getCVCError(expiryDate, _ref) {
  var cardType = _ref.cardType;

  if (!expiryDate) {
    return EMPTY_EXPIRY_DATE;
  }
  if (cardType && expiryDate.length !== cardType.code.length) {
    return INVALID_CVC;
  }
  return;
};
var getZIPError = exports.getZIPError = function getZIPError(zip) {
  if (!zip) {
    return EMPTY_ZIP;
  }
  return;
};